/*
 *  Copyright (C) 2004-2008  gulikoza
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "BaseWindow.h"

BOOL CBaseWindow::RegisterWindow()
{
    WNDCLASSEX wcx; 

    // Fill in the window class structure with default parameters
    wcx.cbSize = sizeof(WNDCLASSEX);							// size of structure
    wcx.style = CS_HREDRAW | CS_VREDRAW;						// redraw if size changes
    wcx.lpfnWndProc = CBaseWindow::stWinMsgHandler;				// points to window procedure
    wcx.cbClsExtra = 0;											// no extra class memory
    wcx.cbWndExtra = 0;											// no extra window memory
    wcx.hInstance = hInstance;									// handle to instance
    wcx.hIcon = LoadIcon(NULL, IDI_APPLICATION);				// predefined app. icon
    wcx.hCursor = LoadCursor(NULL, IDC_ARROW);					// predefined arrow
    hbrBackground = CreateSolidBrush(RGB(16,0,16));				// DON'T FORGET TO DELETE!
    wcx.hbrBackground = hbrBackground;
    wcx.lpszMenuName = NULL;									// name of menu resource
    wcx.lpszClassName = "BaseWindow";							// name of window class
    wcx.hIconSm = LoadIcon(NULL, IDI_APPLICATION);				// small class icon

    // Register the window class.
    return RegisterWindow(&wcx);
}

BOOL CBaseWindow::RegisterWindow(CONST WNDCLASSEX* wcx)
{
	// Register the window class.
	OUTPUTDEBUG("BaseWindow: Registering Class...");

	ZeroMemory(szClassName, sizeof(char)*256);
	strcpy(szClassName, wcx->lpszClassName);

	if (RegisterClassEx(wcx) == 0) {

		DWORD error = GetLastError();
		if(error == 1410L) {
			OUTPUTDEBUG("BaseWindow: WARNING! Class register failed, but create window anyway!");
			OUTPUTDEBUG(szClassName);
			return TRUE;
		}

#ifdef _DEBUG
		LPVOID lpMsgBuf;

		if(!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0,NULL ))
		{
			// Handle the error.
			OutputDebugString("BaseWindow: FormatMessageFailed");
			return FALSE;
		}

		// Display the string.
		OutputDebugString("BaseWindow: ERROR! Registration Failed!");
		OutputDebugString((LPCTSTR)lpMsgBuf);

		// Free the buffer.
		LocalFree(lpMsgBuf);
#endif
		return FALSE;
	} else return TRUE;
}

/*
	You can not initialize the window class with a class method as the window
	procedure unless it is a static method, so the class also needs a static
	message handler that determines which instance of the class is the recipient
	of the message and calls that instance's window procedure.

	See "http://www.gamedev.net/reference/articles/article1810.asp" for more info.
*/
LRESULT CALLBACK CBaseWindow::stWinMsgHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CBaseWindow* pWnd;

	if (uMsg == WM_NCCREATE) {
		// get the pointer to the window from lpCreateParams which was set in CreateWindow
		SetWindowLong(hwnd, GWL_USERDATA, (long)((LPCREATESTRUCT(lParam))->lpCreateParams));
	}

	// get the pointer to the window
	pWnd = GetObjectFromWindow(hwnd);

	// if we have the pointer, go to the message handler of the window
	// else, use DefWindowProc
	if (pWnd) {
		return pWnd->WinMsgHandler(hwnd, uMsg, wParam, lParam);
	} else {
		return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
}

BOOL CBaseWindow::Create()
{
	// Create the window
	RECT rect;

	rect.top = 0;
	rect.left = 0;
	rect.right = 640;
	rect.bottom = 480;

	return Create(WS_VISIBLE | WS_CHILD, &rect);
}

BOOL CBaseWindow::Create(DWORD dwStyles, RECT* rect)
{
	// Create the window
#ifdef _DEBUG
	char debug[256];

	wsprintf(debug, TEXT("BaseWindow: Create szClassName: %s"), szClassName);
	OutputDebugString(debug);
#endif

	// send the this pointer as the window creation parameter
	m_hwnd = CreateWindow(szClassName, szWindowTitle, dwStyles, rect->left, rect->top,
		rect->right - rect->left, rect->bottom - rect->top, hWndParent, NULL, hInstance,
		(void *)this);

#ifdef _DEBUG
	wsprintf(debug, TEXT("BaseWindow: Returning handle...%d (0x%x)\n"), m_hwnd, m_hwnd);
	OutputDebugString(debug);
#endif

	return (m_hwnd != NULL);
}
