/*
 *  Copyright (C) 2004-2008  gulikoza
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* here is the base class for my windows */

#ifndef _BASEWINDOW_H_
#define _BASEWINDOW_H_

#include "DXplay.h"

class CBaseWindow
{
public:
	// many different ways to register
	virtual BOOL RegisterWindow();
	virtual BOOL RegisterWindow(CONST WNDCLASSEX* wcx);

	// static message handler to put in WNDCLASSEX structure
	static LRESULT CALLBACK stWinMsgHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	// just so you can change the window caption...
	void SetWindowTitle(LPCTSTR lpszTitle)
	{
		ZeroMemory(szWindowTitle, sizeof(char)*256);
		strcpy(szWindowTitle, lpszTitle);
	}

	// 2 ways to create
	virtual BOOL Create();
	virtual BOOL Create(DWORD dwStyles, RECT* rect);

	// Need handle
	HWND m_hwnd;

	// some variables
	HBRUSH hbrBackground;
	HWND hWndParent;
	CDXplay *myvideo;

	//contructor
	CBaseWindow(HINSTANCE hInst, CONST WNDCLASSEX* wcx = NULL)
		:m_hwnd(NULL),
		myvideo(NULL),
		hWndParent(0)
	{
		hInstance = hInst;
		if(wcx != NULL) RegisterWindow(wcx);
		else RegisterWindow();
	};

	//destructor
	virtual ~CBaseWindow()
	{

#ifdef _DEBUG
		OutputDebugString("BaseWindow: Calling unregister.");

		if(!UnregisterClass(szClassName, hInstance)) {
			LPVOID lpMsgBuf;
			if(!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
					NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0,NULL ))
			{
				// Handle the error.
				OutputDebugString("BaseWindow: FormatMessageFailed");
				return;
			}
	
			// Display the string.
			OutputDebugString("BaseWindow: ERROR! UnRegistration Failed!");
			OutputDebugString((LPCTSTR)lpMsgBuf);

			// Free the buffer.
			LocalFree(lpMsgBuf);
		} else
			OutputDebugString("BaseWindow: Unregistration successful.");
#else
		UnregisterClass(szClassName, hInstance);
#endif
		OUTPUTDEBUG("BaseWindow: Deleting brush.");
		DeleteObject(hbrBackground);
	}

protected:
	HINSTANCE hInstance;
	char szClassName[256];
	char szWindowTitle[256];

#ifdef _DEBUG
	char debug[256];
#endif

	// the real message handler
	LRESULT CALLBACK WinMsgHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		switch (uMsg)
		{
		case WM_PAINT:
			OUTPUTDEBUG("BaseWindow: Received WM_PAINT");
			PAINTSTRUCT ps;
			HDC hdc;

			hdc = BeginPaint(hwnd, &ps);

			EndPaint(hwnd, &ps);
			break;
		case WM_CLOSE:
			OUTPUTDEBUG("BaseWindow: Received WM_CLOSE");
			DestroyWindow(hwnd);
			break;
		case WM_DESTROY:
			OUTPUTDEBUG("BaseWindow: Received WM_DESTROY");
			PostQuitMessage(0);
			break;
		case WM_P_GRAPHNOTIFY:
			OUTPUTDEBUG("BaseWindow: Received WM_P_GRAPHNOTIFY");
			if((myvideo) && (myvideo->OnGraphEvent())) {
#ifdef _DEBUG
				wsprintf(debug, TEXT("BaseWindow: Calling %d (0x%x)"), hWndParent, hWndParent);
				OutputDebugString(debug);
#endif
				SendMessage(hWndParent, MM_MCINOTIFY, MCI_NOTIFY_SUCCESSFUL, 1);
			}
			break;
		default:
#ifdef _DEBUG
			wsprintf(debug, TEXT("BaseWindow: Received MSG: %x"), uMsg);
			OutputDebugString(debug);
#endif
			return DefWindowProc(hwnd, uMsg, wParam, lParam);
		}

		return 0;
	};

	// returns a pointer the window (stored as the WindowLong)
	inline static CBaseWindow *GetObjectFromWindow(HWND hWnd)
	{
		return (CBaseWindow *)GetWindowLong(hWnd, GWL_USERDATA);
	}
};

#endif
