/*
 *  Copyright (C) 2004-2008  gulikoza
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "DXplay.h"
#include <commctrl.h>
#include "resource.h"

extern HINSTANCE hInstance;

// Compatibility tier (default 1)
TIER tier = T1_REMOVESPLIT;
// Alternate path checked
bool bAltPath = false;
bool bVMR9 = false;
bool bMpeg2Dec = false;
// Path buffer
char szPath[MAX_PATH] = "\0";


BOOL CALLBACK ConfigDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    switch(Message)
    {
		case WM_INITDIALOG:
			SendDlgItemMessage(hwnd, IDC_SLIDER1, TBM_SETRANGE, 0, (LPARAM)MAKELONG(0, 4));
			SendDlgItemMessage(hwnd, IDC_SLIDER1, TBM_SETPOS, TRUE, (LPARAM)tier);
			switch(bAltPath) {
				case true:
					SendDlgItemMessage(hwnd, IDC_CHECK, BM_SETCHECK, BST_CHECKED, 0);
					SetWindowText(GetDlgItem(hwnd, IDC_PATH), szPath);
				break;
				default:
					EnableWindow(GetDlgItem(hwnd, IDC_PATH), false);
					EnableWindow(GetDlgItem(hwnd, IDC_BROWSE), false);
				break;
			}
			switch(bVMR9) {
				case true:
					SendDlgItemMessage(hwnd, IDC_VMR, BM_SETCHECK, BST_CHECKED, 0);
				break;
			}
			switch(bMpeg2Dec) {
				case true:
					SendDlgItemMessage(hwnd, IDC_MPEG2, BM_SETCHECK, BST_CHECKED, 0);
				break;
			}
		// Fall-through
		case WM_HSCROLL:
			tier = (TIER)SendDlgItemMessage(hwnd, IDC_SLIDER1, TBM_GETPOS, 0, 0);
			switch(tier) {
				case T0_REMOVESOURCE:
					SendDlgItemMessage(hwnd, IDC_LEVEL, WM_SETTEXT, 0, (LPARAM)"Level 0: Fastest...video might skip or freeze");
					break;
				case T2_DISCONNECT:
					SendDlgItemMessage(hwnd, IDC_LEVEL, WM_SETTEXT, 0, (LPARAM)"Level 2: Might fix some problems");
					break;
				case T3_REMOVEALL:
					SendDlgItemMessage(hwnd, IDC_LEVEL, WM_SETTEXT, 0, (LPARAM)"Level 3: Select this if you experience problems");
					break;
				case T4_DELETEWND:
					SendDlgItemMessage(hwnd, IDC_LEVEL, WM_SETTEXT, 0, (LPARAM)"Level 4: Safe Mode. Try this last.");
					break;
				default:
					SendDlgItemMessage(hwnd, IDC_LEVEL, WM_SETTEXT, 0, (LPARAM)"Level 1: This is the recommended level (default)");
					break;
			}
		break;
        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case IDOK:
					tier = (TIER)SendDlgItemMessage(hwnd, IDC_SLIDER1, TBM_GETPOS, 0, 0);
					GetWindowText(GetDlgItem(hwnd, IDC_PATH), szPath, MAX_PATH);
					if((bAltPath) && (szPath[0] == '\0')) {
						MessageBox(hwnd, "Please enter file path!", "DXMCI Configure", MB_OK | MB_ICONERROR);
						break;
					}
					DestroyWindow(hwnd);
					PostQuitMessage(0);
                break;
                case IDCANCEL:
					DestroyWindow(hwnd);
					PostQuitMessage(1);
				break;
				case IDC_CHECK: {
					UINT res = IsDlgButtonChecked(hwnd, IDC_CHECK);
					switch(res) {
						case BST_CHECKED:
							bAltPath = true;
							EnableWindow(GetDlgItem(hwnd, IDC_PATH), true);
							EnableWindow(GetDlgItem(hwnd, IDC_BROWSE), true);
						break;
						case BST_UNCHECKED:
							bAltPath = false;
							EnableWindow(GetDlgItem(hwnd, IDC_PATH), false);
							EnableWindow(GetDlgItem(hwnd, IDC_BROWSE), false);
						break;
					}
				}
				break;
				case IDC_VMR: {
					UINT res = IsDlgButtonChecked(hwnd, IDC_VMR);
					switch(res) {
						case BST_CHECKED:
							bVMR9 = true;
						break;
						case BST_UNCHECKED:
							bVMR9 = false;
						break;
					}
				}
				break;
				case IDC_MPEG2: {
					UINT res = IsDlgButtonChecked(hwnd, IDC_MPEG2);
					switch(res) {
						case BST_CHECKED:
							bMpeg2Dec = true;
						break;
						case BST_UNCHECKED:
							bMpeg2Dec = false;
						break;
					}
				}
				break;
				case IDC_BROWSE: {
					OPENFILENAME ofn;       // common dialog box structure
					TCHAR szDirectory[MAX_PATH] = "";

					if(!::GetCurrentDirectory(sizeof(szDirectory) - 1, szDirectory)) {
						MessageBox(hwnd, "Error opening dialog!", "DXMCI Configure", MB_OK | MB_ICONERROR);
						break;
					}

					// Initialize OPENFILENAME
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize = sizeof(ofn);
					ofn.hwndOwner = hwnd;
					ofn.lpstrFile = szPath;

					// Set lpstrFile[0] to '\0' so that GetOpenFileName does not
					// use the contents of szFile to initialize itself.
					ofn.lpstrFile[0] = '\0';
					ofn.nMaxFile = sizeof(szPath);
					ofn.lpstrFilter = "WC4DVD Files (Sc_*.vob)\0Sc_*.vob\0";
					ofn.nFilterIndex = 1;
					ofn.lpstrFileTitle = NULL;
					ofn.nMaxFileTitle = 0;
					ofn.lpstrInitialDir = NULL;
					ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

					// Display the Open dialog box.
					if(GetOpenFileName(&ofn)) {
						char *c = strrchr(szPath, '\\');
						*c = '\0';
						SetWindowText(GetDlgItem(hwnd, IDC_PATH), szPath);
					}

					if(!::SetCurrentDirectory(szDirectory)) {
						MessageBox(hwnd, "Error closing dialog!", "DXMCI Configure", MB_OK | MB_ICONERROR);
						break;
					}
				break;
				}
			} // switch
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

void CALLBACK configure(HWND hwnd, HINSTANCE hinst, LPWSTR lpszCmdLine, int nCmdShow)
{
	HWND hConfigDlg;

	if(!(hConfigDlg=CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG), hwnd, ConfigDlgProc))) {
		MessageBox(NULL, "Error loading configuration dialog!", "DXMCI Configure", MB_OK | MB_ICONERROR);
		return;
	}

	SetWindowPos(hConfigDlg, HWND_TOPMOST, GetSystemMetrics(SM_CXSCREEN) / 2 - 180, GetSystemMetrics(SM_CYSCREEN) / 2 - 90, 0, 0, SWP_NOSIZE);
	ShowWindow(hConfigDlg, TRUE);

	BOOL bRet;
	MSG Msg;
	while((bRet=GetMessage(&Msg, NULL, 0, 0)) > 0)
	{
		if(!IsDialogMessage(hConfigDlg, &Msg))
		{
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
	}

	if(bRet == -1) {	// Error
		MessageBox(NULL, "Error saving configuration! (0x01)", "DXMCI Configure", MB_OK | MB_ICONERROR);
		return;
	}

	if(Msg.wParam == 1) // Cancel
		return;
	else {				// OK

		HANDLE hRead, hWrite;
		DWORD dwReadBytes, dwWriteBytes;

		char *buffer;
		int i, off[5];

		// Offsets of the variables
		off[0] = ((int)&tier - (int)hInstance);
		off[1] = ((int)&bAltPath - (int)hInstance);
		off[2] = ((int)szPath - (int)hInstance);
		off[3] = ((int)&bVMR9 - (int)hInstance);
		off[4] = ((int)&bMpeg2Dec - (int)hInstance);

		hRead = CreateFile(TEXT("dxmci.dll"),	// file to open
					GENERIC_READ,				// open for reading
					FILE_SHARE_READ,			// share for reading
					NULL,						// default security
					OPEN_EXISTING,				// existing file only
					FILE_ATTRIBUTE_NORMAL,		// normal file
					NULL);						// no attr. template

		if (hRead == INVALID_HANDLE_VALUE) {
			MessageBox(NULL, "Error saving configuration! (0x02)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			return;
		}

		hWrite = CreateFile(TEXT("$temp$.dll"),	// file to create
					GENERIC_WRITE,				// open for writing
					0,							// do not share
					NULL,						// default security
					CREATE_ALWAYS,				// overwrite existing
					FILE_ATTRIBUTE_NORMAL,		// normal file
					NULL);						// no attr. template

		if (hWrite == INVALID_HANDLE_VALUE) {
			MessageBox(NULL, "Error saving configuration! (0x03)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			CloseHandle(hRead);
			return;
		}

		i = SetFilePointer(hRead, 0, 0, FILE_END);
		SetFilePointer(hRead, 0, 0, FILE_BEGIN);

		buffer = new char[i];
		if(!buffer) {
			MessageBox(NULL, "Error saving configuration! (0x04)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			CloseHandle(hRead);
			CloseHandle(hWrite);
			DeleteFile(TEXT("$temp$.dll"));
			return;
		}

		while(i > 0) {
			if(!ReadFile(hRead, buffer, i, &dwReadBytes, NULL)) {
				MessageBox(NULL, "Error saving configuration! (0x05)", "DXMCI Configure", MB_OK | MB_ICONERROR);
				CloseHandle(hRead);
				CloseHandle(hWrite);
				DeleteFile(TEXT("$temp$.dll"));
				return;
			}

			i -= dwReadBytes;
			// Change the variables inside the buffer
			if((off[0] >= 0) && (off[0] < (int)dwReadBytes))
				*(buffer + off[0]) = tier;
			if((off[1] >= 0) && (off[1] < (int)dwReadBytes))
				*(buffer + off[1]) = bAltPath;
			if((off[2] >= 0) && (off[2] < (int)dwReadBytes))
				strcpy(buffer + off[2], szPath);
			if((off[3] >= 0) && (off[3] < (int)dwReadBytes))
				*(buffer + off[3]) = bVMR9;
			if((off[4] >= 0) && (off[4] < (int)dwReadBytes))
				*(buffer + off[4]) = bMpeg2Dec;

			off[0] -= dwReadBytes;
			off[1] -= dwReadBytes;
			off[2] -= dwReadBytes;
			off[3] -= dwReadBytes;
			off[4] -= dwReadBytes;

			if((!WriteFile(hWrite, buffer, dwReadBytes, &dwWriteBytes, NULL)) || (dwReadBytes > dwWriteBytes)) {
				MessageBox(NULL, "Error saving configuration! (0x06)", "DXMCI Configure", MB_OK | MB_ICONERROR);
				CloseHandle(hRead);
				CloseHandle(hWrite);
				DeleteFile(TEXT("$temp$.dll"));
				return;
			}
		}

		// The file is copied OK and the variable changed
		CloseHandle(hRead);
		CloseHandle(hWrite);
		delete[] buffer;

		hWrite = CreateFile(TEXT("$temp$.bat"),	// file to create
					GENERIC_WRITE,				// open for writing
					0,							// do not share
					NULL,						// default security
					CREATE_ALWAYS,				// overwrite existing
					FILE_ATTRIBUTE_NORMAL,		// normal file
					NULL);

		if (hWrite == INVALID_HANDLE_VALUE) {
			MessageBox(NULL, "Error saving configuration! (0x07)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			DeleteFile(TEXT("$temp$.dll"));
			return;
		}

		// Create the bat file :-)
		wsprintf(buffer, "@echo off\r\n:DELETE\r\ndel dxmci.dll\r\nif exist dxmci.dll goto DELETE\r\nren $temp$.dll dxmci.dll\r\ndel $temp$.bat");
		if((!WriteFile(hWrite, buffer, strlen(buffer), &dwWriteBytes, NULL)) || (strlen(buffer) > dwWriteBytes)) {
			MessageBox(NULL, "Error saving configuration! (0x08)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			CloseHandle(hWrite);
			DeleteFile(TEXT("$temp$.dll"));
			DeleteFile(TEXT("$temp$.bat"));
		}

		CloseHandle(hWrite);
		// Now execute it and exit
		dwReadBytes = WinExec("$temp$.bat", SW_HIDE);
		if(dwReadBytes < 32) {
			MessageBox(NULL, "Error saving configuration! (0x09)", "DXMCI Configure", MB_OK | MB_ICONERROR);
			DeleteFile(TEXT("$temp$.dll"));
			DeleteFile(TEXT("$temp$.bat"));
		}
	}
}
