/*
 *  Copyright (C) 2004-2008  gulikoza
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// dxmci.cpp : Defines the entry point for the DLL application.
//

#include "dxmci.h"
#include <math.h>

#define SAFE_DELETE(p) { if(p) delete p; p = NULL; }

BOOL APIENTRY DllMain( HANDLE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	if(ul_reason_for_call == DLL_PROCESS_ATTACH) {

		OUTPUTDEBUG("Dxmci.dll: Library loaded...initializing parameteres");

		// Initialize some data...
		cominit = 0;
		mainWnd = NULL;
		m_hwnd = m_parent = 0;

		hInstance = (HINSTANCE)hModule;

	}
	else if(ul_reason_for_call == DLL_PROCESS_DETACH) {

		OUTPUTDEBUG("Dxmci.dll: Library unloading.");

		// We can't delete DShowFile here because it will unload DShow dlls and hang
		if(mainWnd) {
			OUTPUTDEBUG("Dxmci.dll: Will delete our child window now");

			DestroyWindow(mainWnd->m_hwnd);
			SAFE_DELETE(mainWnd)
		}

		// Not safe...so we'll just leave it on
		/*CoUninitialize();*/
	}

    return TRUE;
}

MCIDEVICEID WINAPI mciGetDeviceIDD(LPCTSTR lpszDevice)
{
	OUTPUTDEBUG("Dxmci.dll: mciGetDeviceIDA called. Returning 1");
	return 1;
}

BOOL WINAPI mciGetErrorStringD(DWORD fdwError, LPTSTR lpszErrorText, UINT cchErrorText)
{
	OUTPUTDEBUG("Dxmci.dll: mciGetErrorStringA called. Returning 1");
	return 1;
}

MCIERROR WINAPI mciSendCommandD(MCIDEVICEID mciId,UINT uMsg,DWORD dwParam1,DWORD dwParam2)
{

#ifdef _DEBUG
	char ModuleName[260];
	char debug[512];

	wsprintf(debug, TEXT("Dxmci.dll: mciSendCommandA(0x%x, 0x%x, 0x%x, 0x%x) called."), mciId, uMsg, dwParam1, dwParam2);
	OutputDebugString(debug);
	switch(dwParam1) {
		case MCI_NOTIFY: OutputDebugString("MCI_NOTIFY flag..."); break;
		case MCI_WAIT: OutputDebugString("MCI_WAIT flag..."); break;
		default: OutputDebugString("Unknown flag..."); break;
	}
#endif

	switch(uMsg) {
	case MCI_OPEN: {

		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_OPEN");
		MCI_OVLY_OPEN_PARMS *parms;
		parms = (MCI_OVLY_OPEN_PARMS*)dwParam2;

		if((dwParam1 & MCI_OVLY_OPEN_PARENT) == MCI_OVLY_OPEN_PARENT) {

#ifdef _DEBUG
			GetWindowModuleFileName(parms->hWndParent, ModuleName, 259);
			wsprintf(debug, TEXT("Dxmci.dll: Parent window given: hWndParent: %d (0x%x), exe: %s"),
					 parms->hWndParent, parms->hWndParent, ModuleName);
			OutputDebugString(debug);
#endif
			// Initialize COM if necessary
			if(cominit == 0) {
				CoInitialize(NULL);
				cominit = TRUE;
			}

			// Create new direct show file
			if(!DShowFile) {
				DShowFile = new CDXplay;
			}

			if(!DShowFile) {
				OUTPUTDEBUG("Dxmci.dll: DShowFile creation failed.");
				return MCIERR_INVALID_DEVICE_NAME;
			}

			// Create Window
			if(!mainWnd) {
				OUTPUTDEBUG("Dxmci.dll: Creating new window.");

				mainWnd = new CBaseWindow(hInstance);
				if(!mainWnd) {
					OUTPUTDEBUG("Dxmci.dll: Window creation failed.");
					return MCIERR_INVALID_DEVICE_NAME;
				}
			}

			mainWnd->SetWindowTitle("WC4");
			m_parent = parms->hWndParent;
			mainWnd->hWndParent = m_parent;

			// create the window
			if(!mainWnd->m_hwnd) {
				OUTPUTDEBUG("Dxmci.dll: Calling create.");
				mainWnd->Create();

#ifdef _DEBUG
				wsprintf(debug, TEXT("Dxmci.dll: new window handle: %d"), mainWnd->m_hwnd);
				OutputDebugString(debug);
#endif

				m_hwnd = mainWnd->m_hwnd;
			}
		}

		if(m_hwnd) {
			mainWnd->myvideo = DShowFile;
			ShowWindow(m_hwnd, SW_SHOW);
		}

#ifdef _DEBUG
		wsprintf(debug, TEXT("Dxmci.dll: Launching movie in window: %d"), m_hwnd);
		OutputDebugString(debug);
#endif

		if(!DShowFile->LoadMediaFile(parms->lpstrElementName)) {
			MessageBox(NULL, "Error loading video!", "DXMCI Error", MB_OK | MB_ICONERROR);
			return MCIERR_FILE_NOT_FOUND;
		}

#ifdef _DEBUG
		wsprintf(debug, TEXT("Dxmci.dll: Setting parent movie window to %d"), m_hwnd);
		OutputDebugString(debug);
#endif

		DShowFile->SetParentWindow(m_hwnd);
		//SetFocus(m_hwnd);

		// Set expected DeviceID
		parms->wDeviceID = 1;

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;		// File opened!
	}
	case MCI_CLOSE: {

		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_CLOSE");

		DShowFile->Stop();
		ShowWindow(m_hwnd, SW_HIDE);
		//SendMessage(mainWnd->hWndParent, MM_MCINOTIFY, MCI_NOTIFY_ABORTED, 1);

		// We have to do this here...
		if(tier > T0_REMOVESOURCE) {
			if(mainWnd) mainWnd->myvideo = NULL;
			SAFE_DELETE(DShowFile)
		}

		if(tier >= T4_DELETEWND) {
			if(mainWnd) {
				OUTPUTDEBUG("Dxmci.dll: Will delete our child window now");

				DestroyWindow(mainWnd->m_hwnd);
				SAFE_DELETE(mainWnd)
			}
		}

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;	// Success!
	}
	case MCI_PLAY: { 

		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_PLAY");

		if(!DShowFile->Run()) {
			MessageBox(NULL, "Error while playing video!", "DXMCI Error", MB_OK | MB_ICONERROR);
			return MCIERR_UNRECOGNIZED_COMMAND;
		}

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;	// Success!
	}
	case MCI_PAUSE:	{

		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_PAUSE");
		
		DShowFile->Pause();

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;	// Success!
	}
	case MCI_STATUS: {
		
		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_STATUS");

		MCI_STATUS_PARMS *parms;
		parms = (MCI_STATUS_PARMS*)dwParam2;
		//SetFocus(mainWnd->hWndParent);

		if((dwParam1 & MCI_STATUS_ITEM) == MCI_STATUS_ITEM)  {

#ifdef _DEBUG
			wsprintf(debug, TEXT("Dxmci.dll: Clinet requesting extra status: 0x%x"), parms->dwItem);
			OutputDebugString(debug);
#endif

			if((parms->dwItem & MCI_OVLY_STATUS_HWND) == MCI_OVLY_STATUS_HWND) {
				parms->dwReturn = MAKELONG(m_hwnd, 0);
#ifdef _DEBUG
				wsprintf(debug, TEXT("Dxmci.dll: Client requesting MCI_OVLY_STATUS_HWND, giving %d (0x%x)"), m_hwnd, m_hwnd);
				OutputDebugString(debug);
#endif
			}
		}

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;	// Success!
	}
	case MCI_LOAD: {

		OUTPUTDEBUG("Dxmci.dll: mci command: MCI_LOAD");

		MCI_OVLY_LOAD_PARMS *parms;
		parms = (MCI_OVLY_LOAD_PARMS*)dwParam2;

#ifdef _DEBUG	
		wsprintf(debug, TEXT("Loading filename...%s"), parms->lpfilename);
		OutputDebugString(debug);
#endif

		if(tier >= T4_DELETEWND) {
			SAFE_DELETE(DShowFile)

			// Create new direct show file
			if(!DShowFile)
				DShowFile = new CDXplay;

			if(!DShowFile) {
				OUTPUTDEBUG("Dxmci.dll: DShowFile creation failed.");
				return MCIERR_INVALID_DEVICE_NAME;
			}

			mainWnd->myvideo = DShowFile;
		}

		ShowWindow(m_hwnd, SW_SHOW);

		if(!DShowFile->LoadMediaFile(parms->lpfilename)) {
			MessageBox(NULL, "Error loading video!", "DXMCI Error", MB_OK | MB_ICONERROR);
			return MCIERR_FILE_NOT_FOUND;
		}

		DShowFile->SetParentWindow(m_hwnd);
		SetFocus(m_parent);
		//SetFocus(m_hwnd);

		OUTPUTDEBUG("Dxmci.dll: mci command complete.");

		return 0;
	}
	default:
		OUTPUTDEBUG("Dxmci.dll: Unimplemented mci command.");
		return 0;
	}
}

MMRESULT WINAPI dxMidiGetVolume(HMIDIOUT hmo, LPDWORD pdwVolume)
{
	DShowFile->GetVolume((signed long*)pdwVolume);

	return 0;
}

MMRESULT WINAPI dxMidiSetVolume(HMIDIOUT hmo, DWORD dwVolume)
{
	long db;

	double volume;

	volume = (double) dwVolume / 4294967295;

	if(volume > 0)
		volume = (double)log(volume);
	else
		volume = -10;
	
	volume *= 1000;

	db = (long)volume;

	DShowFile->SetVolume(db);

	return 0;
}

#define STAT_SEEKING 0
#define STAT_PLAYING 1
#define STAT_STOPPED 2

char *status[] = { "seeking", "playing", "stopped" };

MCIERROR WINAPI dxMidiSendString(LPCTSTR lpstrCommand, LPTSTR lpstrReturnString, UINT uReturnLength, HWND hwndCallback)
{
	char command, *pos;
	char file[_MAX_PATH+1];

	static bool seeking;

	command = lpstrCommand[0];

	// check only first character, it's unique
	switch(command) {
		case 'o':	// "open <fullpath> type sequencer alias MUSIC"

			// split full command into pathname

			strncpy(file, lpstrCommand + 5, _MAX_PATH);
			pos = strrchr(file, '\\') + 1;

			while(*pos != ' ') pos++;

			*pos = 0;

			// Load our file
			DShowFile->Delete();
			DShowFile->LoadMediaFile(file);

			break;
		case 'p':	// "play MUSIC from 0"
			DShowFile->Run();
			seeking = 1;
			break;
		case 's':	// "status MUSIC mode"

			// if a segment has been defined, and it was "seeking",
			// check whether it's done seeking yet
			if (seeking && DShowFile->IsRunning())
				seeking = false;

			// then return the appropriate result
			strcpy(lpstrReturnString, status
				[
				  seeking ?	STAT_SEEKING :
				  DShowFile->IsRunning() ? STAT_PLAYING : STAT_STOPPED
				] );
			break;
		case 'c':	// "close all"
			DShowFile->Stop();
			break;
	}

	return 0;
}

MMRESULT WINAPI dxmcKillEvent(UINT uTimerID)
{
	if(!h_winmm)
		h_winmm = LoadLibrary("winmm.dll");
	if(!p_timeKillEvent)
		p_timeKillEvent = (pfntimeKillEvent)GetProcAddress(h_winmm, "timeKillEvent");
	return p_timeKillEvent(uTimerID);
}

DWORD WINAPI dxmcGetTime(void)
{
	if(!h_winmm)
		h_winmm = LoadLibrary("winmm.dll");
	if(!p_timeGetTime)
		p_timeGetTime = (pfntimeGetTime)GetProcAddress(h_winmm, "timeGetTime");
	return p_timeGetTime();
}

MMRESULT WINAPI joyGetDevCapsD(UINT uJoyID, LPJOYCAPSA pjc, UINT cbjc)
{
	if(!h_winmm)
		h_winmm = LoadLibrary("winmm.dll");
	if(!p_joyGetDevCapsA)
		p_joyGetDevCapsA = (pfnjoyGetDevCapsA)GetProcAddress(h_winmm, "joyGetDevCapsA");
	return p_joyGetDevCapsA(uJoyID, pjc, cbjc);
}

MMRESULT WINAPI joyGetPosDx(UINT uJoyID, LPJOYINFOEX pji)
{
	if(!h_winmm)
		h_winmm = LoadLibrary("winmm.dll");
	if(!p_joyGetPosEx)
		p_joyGetPosEx = (pfnjoyGetPosEx)GetProcAddress(h_winmm, "joyGetPosEx");
	return p_joyGetPosEx(uJoyID, pji);
}
